#!/usr/bin/env Rscript
args = commandArgs(trailingOnly=TRUE)

# collect and summarize quast stats from milonga

library(dplyr)

# thrsholds positivkontrolle
threshold_positivkontrolle_coverage.depth = 30
threshold_positivkontrolle_X..contigs = 1
threshold_positivkontrolle_reference.coverage = 0.98
threshold_positivkontrolle_reference.similarity = 0.98

# TODO read from file?
# TODO print
print(paste(
"threshold_positivkontrolle_coverage.depth", threshold_positivkontrolle_coverage.depth,
"threshold_positivkontrolle_X..contigs",threshold_positivkontrolle_X..contigs,
"threshold_positivkontrolle_reference.coverage", threshold_positivkontrolle_reference.coverage,
"threshold_positivkontrolle_reference.similarity",threshold_positivkontrolle_reference.similarity
))
# expects milonga standard output
# expects quast vs. reference output

# test if there is at least one argument: if not, return an error
if (length(args)==0) {
  stop("At least one argument must be supplied (input file).n", call.=FALSE)
} #else if (length(args)==1) {
  # default output file
  #args[2] = "out.txt"
#}
# TODO make argument
quastdir <- args[1] 
print(paste("quastdir",quastdir))

#quastdir <- "/cephfs/abteilung4/Datentransfer/MinIon_Network/20211008_1722_MN23248_AGY494_5ac7191c/Positivkontrolle/flye/quast"
sampledir <- dirname(dirname(quastdir))
#sampledir <- "/cephfs/abteilung4/Datentransfer/MinIon_Network/20211008_1722_MN23248_AGY494_5ac7191c/Positivkontrolle"

# outfile
summary_file <- file.path(quastdir,"summary_qc_positivecontrol.tsv")

sample <- basename(sampledir)
rundir <- dirname(sampledir)
run <- basename(rundir)


#/cephfs/abteilung4/Datentransfer/MinIon_Network/20211008_1722_MN23248_AGY494_5ac7191c/Positivkontrolle/reports/summary_qc.tsv


reference_info.file <- "/cephfs/abteilung4/Datentransfer/MinIon_Network/20211008_1722_MN23248_AGY494_5ac7191c/Positivkontrolle/flye/reference_info.txt"
# TODO make path relative to analysis
if(file.exists(reference_info.file)){
  reference_info <- scan(reference_info.file, what = character(), sep = "\n")  
} else{
  reference_info <- NA
}  

message("sampledir: ",sampledir)
summary_qc.tsv <- file.path(sampledir,"reports/summary_qc.tsv")
message("summary_qc.tsv: ",summary_qc.tsv)
stopifnot(file.exists(summary_qc.tsv))
summary_qc <- read.delim(summary_qc.tsv, stringsAsFactors = F)



#assembler <- "flye" # TODO make flexible
assembler <- basename(dirname(quastdir))
#quastdir <- file.path(sampledir,assembler,"quast")
stopifnot(dir.exists(quastdir))

# ---
quast_stats.file <- file.path(quastdir,"transposed_report.tsv")

if (file.exists(quast_stats.file)){
  quast_stats <- read.delim(quast_stats.file, stringsAsFactors = F)
} else{
  stop("file ",quast_stats.file, " does NOT exist")
} 


# snps NOTE name depends on name of input fasta file
#snps.file <- file.path(quastdir,"contigs_reports","minimap_output",paste0(sample,".used_snps.gz")) 
#snps.file <- file.path(quastdir,"contigs_reports","minimap_output",paste0("assembly",".used_snps.gz")) 

snps.file <- list.files(file.path(quastdir,"contigs_reports","minimap_output"), pattern = ".used_snps.gz", full.names = T )

if (length(snps.file) == 0){
  warning("File ",snps.file," does NOT exist")
  snps <- NA
  snp_pos <- NA
} else if (length(snps.file) > 1){
  stop("More than one snp file found in folder\n", snps.file,"\n Please run quast on single fasta file")  
} else {
  snps_table <- read.delim(snps.file, header = F, col.names = c("Reference","contig","POS","REF","ALT","SCORE"))
  snps <- nrow(snps_table)
  snp_pos <- snps_table %>% select(POS) %>% unlist %>% sort %>% paste(.,collapse = ",")
} 



if(file.exists(snps.file)){
  snps_table <- read.delim(snps.file, header = F, col.names = c("Reference","contig","POS","REF","ALT","SCORE"))
  snps <- nrow(snps_table)
  snp_pos <- snps_table %>% select(POS) %>% unlist %>% sort %>% paste(.,collapse = ",")
} else{
  warning("File ",snps.file," does NOT exist")
  snps <- NA
  snp_pos <- NA
} 



# return
summary_table <- data.frame(
  sample = sample,
  run = run,
  assembler = assembler,
  rundir = rundir,
  reference_name = reference_info,
  # reads_long
  coveragedepth_longreads = summary_qc %>% filter(sample == sample) %>% select(long_read_coverage) %>% unlist(),
  total_length = quast_stats$Total.length.....0.bp.,
  count_contigs = quast_stats$X..contigs.....0.bp.,
  reference_coverage = quast_stats$Genome.fraction....,
  # reference_similarity # does not exist
  # other definition: 
  reference_similarity = quast_stats$Total.aligned.length/quast_stats$Reference.length,
  #pass = pass_positivkontrolle,
  mismatch_frequency = quast_stats$X..mismatches.per.100.kbp,
  indel_frequency = quast_stats$X..indels.per.100.kbp,
  snps = snps,
  snp_pos = snp_pos)


# add pass/info
# pass check
# pass criterion
pass_positivkontrolle <- ifelse(  summary_table$coveragedepth_longreads >= threshold_positivkontrolle_coverage.depth &
                                    summary_table$count_contigs == threshold_positivkontrolle_X..contigs &
                                    summary_table$reference_coverage >= threshold_positivkontrolle_reference.coverage &
                                    summary_table$reference_similarity >= threshold_positivkontrolle_reference.similarity,"PASS","FAIL")


summary_table$qc_evaluation <- pass_positivkontrolle

# print
print(summary_table)

# export
write.table(summary_table,summary_file,sep = "\t",row.names = F, quote = F)

# 
print(paste("Done"))
