# parse paf and remove all best self alignments

library(dplyr)

# define input
#input_filename <- "/home/carlus/BfR/Projects/milonga_testdata/self_alignment.paf"
input_filename <- snakemake@input[["self_alignment"]]
# parameter
sample <- snakemake@params[["sample"]]
assembly <- snakemake@params[["assembly"]]
# define output

alignment_filtered.file <- snakemake@output[["filtered_alignment"]]
summary.file <- snakemake@output[["selfalignment_summary"]]


  
# read in alignments
alignments = read.table(input_filename, stringsAsFactors = F, fill = T)

# set column names
# PAF IS ZERO-BASED - CHECK HOW CODE WORKS
colnames(alignments)[1:12] = c("queryID","queryLen","queryStart","queryEnd","strand","refID","refLen","refStart","refEnd","numResidueMatches","lenAln","mapQ")


# alignment on same contig 

same_contigs <- alignments %>% filter(queryID == refID)
contigs <- unique(same_contigs$queryID)

filtered_alignment_same <- do.call(rbind,lapply(contigs, function(mycontig) {
  # dummy
  #remaining_alignments <- same_contigs %>% filter(queryID == mycontig)
  
  remaining_alignments <- same_contigs %>% filter(queryID == mycontig)  %>% filter(lenAln != max(lenAln))
  # 
  return(remaining_alignments)
}))

# alignment between different contigs
filtered_alignment_diffcontigs <- alignments %>% filter(queryID != refID)

# merge
filtered_alignment_all <- rbind(filtered_alignment_same,filtered_alignment_diffcontigs)


# write out
write.table(filtered_alignment_all, alignment_filtered.file, sep = "\t", row.names = F, quote = F)

# summary

summary <- data.frame(
  sample = sample,
  assembly = assembly,
  misalignments = nrow(filtered_alignment_all),
  misalignments_same = nrow(filtered_alignment_same),
  misalignments_diff = nrow(filtered_alignment_diffcontigs),
  misalignment_length = sum(filtered_alignment_all$lenAln)
)


# write out 2
write.table(summary,summary.file, sep = "\t", row.names = F, quote = F)



